;+
; NAME:
;   SPLIT_GRAVITY
;
; PURPOSE:
;   Splits one (or more, see below) GRAVITY OIFITS in two files, one for the SCience
;   detector, the other for the FringeTracker detector. Optionnally,
;   arrange headers so that Dates and Times in various tables are
;   consistent with each other.
;   Can be called with a serie of files instead of just one.
;   Calls a file chooser if the file(s) is(are) not provided.
;   With a serie of files, can otionnaly merge the data into only one
;   FT and one SC file.
;
; CALLING SEQUENCE:
;   SPLIT_GRAVITY,files [, /by_polar], [,/align_times] [,/merge] [,/verbose]
;
; INPUTS:
;   files :  vector of oifits file names. If absent, call
;   dialog_pickfile.
;
; CAVEATS:
;   only crude tests are applied to insure that the input files are
;   indeed genuine files out of the GRAVITY pipeline, not non-gravity
;   files or modified GRAVITY OIFITS. In the case of concatenation
;   (/MERGE) input files must refer TO THE SAME OBJECT.
;
; KEYWORD Parameters:
;   BY_POLAR: if set, splitting will be between 2 polarisations and
;   not FT/SC. 
;   MERGE: if set, provided filename is an array of size > 1, merges
;   all the files in two, one for the SCience,  the other for the FT.
;   ALIGN_TIMES: if set, provided the file is issued from a single
;   observation, makes MJD and TIME consistent between T3, V2 and VIS
;   tables.
;
;-

PRO SUB_SPLIT_GRAVITY,ntot,ifile,filename, align_times=align_times, merge=merge, silent=silent

COMMON  COMMONFORSPLITGRAVITY,mergedname,key_string
  doMerge=(keyword_set(merge) and ntot gt 1)
  doAlign=keyword_set(align_times)

  fits_info,filename, n_ext=next, extname=extname, /silent
; shorten extname
  extname=strtrim(extname,2)
;if name contains a path, be sure to treat it correctly
  name=file_basename(filename)
  dir=file_dirname(filename)
  c=path_sep()
;read first header
  hdu0=mrdfits(filename,0,header0,silent=silent)

; check some values in the file:
; must have both SC and FT, i.e., 2 OI_WAVELENGTH of these names.
; must have exactly 2 OI_T3, 2 OI_VIS2, 2 OI_VIS.
; otherwise it is not a genuine "atomic" gravity file and this
; procedure would probably make a mess.
  nt3=0
  nvis=0
  nvis2=0
  ninst=0
  sc=0
  ft=0
  for i=1,next do begin
     if ( extname[i] EQ "OI_T3") then nt3++
     if ( extname[i] EQ "OI_VIS") then nvis++;
     if ( extname[i] EQ "OI_VIS2") then nvis2++;
     if ( extname[i] EQ "OI_WAVELENGTH") then ninst++;
     if ( extname[i] EQ "OI_VIS2" or extname[i] EQ "OI_VIS" or extname[i] EQ "OI_T3") then begin
        hdu= mrdfits(filename,i,header,silent=silent)
        headernlines=n_elements(header)
        found=stregex(header,"INSNAME = 'GRAVITY.*"+key_string[0]+".*") ; FT or P1
        if (total(found) eq -1* headernlines) then sc++ else ft++;
     endif
  endfor
  if (nt3 lt 2 or nvis lt 2 or nvis2 lt 2) then message,"Non-standard GRAVITY file, aborting."
  if (sc lt 3 or ft lt 3) then message,"Non-standard GRAVITY file, aborting."

  if (doMerge and ifile eq 0) then mergedname="MERGE_"+name

  if (doMerge) then begin
     sc_name=dir+c+key_string[1]+"_"+mergedname
     ft_name=dir+c+key_string[0]+"_"+mergedname
     if (ifile eq 0) then mwrfits,hdu0,FT_NAME,header0,/create,silent=silent
     if (ifile eq 0) then mwrfits,hdu0,SC_NAME,header0,/create,silent=silent
  endif else begin
     sc_name=dir+c+key_string[1]+"_"+name
     ft_name=dir+c+key_string[0]+"_"+name
     mwrfits,hdu0,FT_NAME,header0,/create,silent=silent
     mwrfits,hdu0,SC_NAME,header0,/create,silent=silent
  endelse

  if (doAlign) then begin
; find the T3 and set times to the values of the first line. Remember
; these values for the VIS and VIS2 files.
     for i=1,next do begin
        if ( extname[i] EQ "OI_T3") then begin
           ; this is the first T3. get all the times from it.
           hdu= mrdfits(filename,i,header,silent=silent)
           TIME=hdu[0].time
           MJD=hdu[0].mjd
           break
        endif
     endfor
  endif

;proceed, and convert times and MJD if necessary
  for i=1,next do begin
     if ( extname[i] EQ "OI_TARGET" or extname[i] EQ "OI_ARRAY") then begin
        hdu= mrdfits(filename,i,header,silent=silent)
        if (~doMerge or ifile eq 0) then begin
           mwrfits,hdu,SC_NAME,header,silent=silent
           mwrfits,hdu,FT_NAME,header,silent=silent
        endif
     endif else if ( extname[i] EQ "OI_WAVELENGTH") then begin
        hdu= mrdfits(filename,i,header,silent=silent)
        headernlines=n_elements(header)
        found=stregex(header,"INSNAME = 'GRAVITY.*"+key_string[0]+".*") ; FT or P1
        if (~doMerge or ifile eq 0) then begin
           if (total(found) eq -1* headernlines) then mwrfits,hdu,SC_NAME,header,silent=silent else mwrfits,hdu,FT_NAME,header,silent=silent
        endif
     endif else begin
        hdu= mrdfits(filename,i,header,silent=silent)
        if doAlign and (extname[i] EQ "OI_VIS" or extname[i] EQ "OI_VIS2" or extname[i] EQ "OI_T3" ) then begin
           hdu.time=TIME
           hdu.mjd=MJD
        endif
        headernlines=n_elements(header)
                                ; find "FLAG" columns as they should
                                ; be written in logical mode, not in
                                ; byte mode
        w=where(tag_names(hdu) eq "FLAG", count)
        w++                                     ; since mwrfits counts start at 1.
        isft=stregex(header,"INSNAME = 'GRAVITY.*"+key_string[0]+".*") ; FT or P1 should work for split & comb
        issc=stregex(header,"INSNAME = 'GRAVITY.*"+key_string[1]+".*") ; SC or P2
        if (count gt 0) then begin 
           if (total(issc) ne -1* headernlines) then mwrfits,hdu,SC_NAME,header,LOGICAL_COLS=w,silent=silent
           if (total(isft) ne -1* headernlines) then mwrfits,hdu,FT_NAME,header,LOGICAL_COLS=w,silent=silent
        endif else begin
           if (total(issc) ne -1* headernlines) then mwrfits,hdu,SC_NAME,header,silent=silent
           if (total(isft) ne -1* headernlines) then mwrfits,hdu,FT_NAME,header,silent=silent
        endelse
     endelse
  endfor
END
PRO SPLIT_GRAVITY,filename, align_times=align_times, merge=merge, by_polar=by_polar, verbose=verbose

silent=~keyword_set(verbose)

COMMON COMMONFORSPLITGRAVITY,mergedname,key_string
  if keyword_set(by_polar) then key_string=['P1','P2'] else key_string=['FT','SC']
  if (n_params() eq 0) then filename=dialog_pickfile(/mult)
  ntot=n_elements(filename)
  mergedname=""
  for i=0,ntot-1 do SUB_SPLIT_GRAVITY,ntot,i,filename[i], align_times=align_times, merge=merge, silent=silent
END
